<?php

namespace Intuit\Response;

use Illuminate\Support\Collection;

/**
 * Stores incoming or outgoing message data for a Slack API call.
 */
class Webhook implements \ArrayAccess, \JsonSerializable
{
	/**
	 * @var Collection The response data.
	 */
	protected Collection $data;

	/**
	 * Creates a new payload object.
	 *
	 * @param array $data The payload data.
	 */
	public function __construct(array $data,bool $key=FALSE)
	{
		$this->data = collect($key ? ['payload'=>$data ] : $data);
	}

	/**
	 * @return string
	 */
	public function __toString()
	{
		return $this->toJson();
	}

	/**
	 * Creates a response object from a JSON message.
	 *
	 * @param string $json A JSON string.
	 * @return Webhook The parsed response.
	 */
	public static function fromJson($json): self
	{
		$data = json_decode((string)$json,true);

		if (json_last_error() !== JSON_ERROR_NONE || (! is_array($data))) {
			throw new \UnexpectedValueException('Invalid JSON message:'.serialize($data));
		}

		return new static($data);
	}

	/* INTERFACES */

	/**
	 * @param mixed $offset
	 * @param mixed $value
	 */
	public function offsetSet($offset, $value)
	{
		if (is_null($offset)) {
			$this->data[] = $value;

		} else {
			$this->data[$offset] = $value;
		}
	}

	/**
	 * @param mixed $offset
	 * @return bool
	 */
	public function offsetExists($offset)
	{
		return isset($this->data[$offset]);
	}

	/**
	 * @param mixed $offset
	 */
	public function offsetUnset($offset)
	{
		unset($this->data[$offset]);
	}

	/**
	 * @param mixed $offset
	 * @return null
	 */
	public function offsetGet($offset)
	{
		return $this->data[$offset] ?? NULL;
	}

	/**
	 * @return array
	 */
	public function jsonSerialize()
	{
		return $this->data;
	}

	/* METHODS */

	/**
	 * Return the events of a specific type
	 * @param string $type
	 * @return Collection
	 */
	public function event(string $type): Collection
	{
		return collect($this->data->get($type));
	}

	/**
	 * Gets the payload data.
	 *
	 * @return array The payload data.
	 */
	public function getData()
	{
		return $this->data;
	}

	/**
	 * Serializes the payload to a JSON message.
	 *
	 * @return string A JSON message.
	 */
	public function toJson(): string
	{
		return json_encode($this->data,true);
	}

	/**
	 * Return the event types
	 *
	 * @return Collection
	 */
	public function types(): Collection
	{
		return $this->data->keys();
	}
}